package com.dcit.danger;

import java.util.ArrayList;
import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Contact;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;


@Configuration
@EnableSwagger2
@ConditionalOnProperty(prefix = "swagger",value = {"enable"},havingValue = "true")
public class SwaggerConfig {

	@Bean
	public Docket createRestApi(){
		
		ParameterBuilder ticketPar = new ParameterBuilder();
	    List<Parameter> pars = new ArrayList<Parameter>();  
		ticketPar.name("Authorization").description("user ticket")
		.modelRef(new ModelRef("string")).parameterType("header") 
		.required(false).build(); //header中的ticket参数非必填，传空也可以
		pars.add(ticketPar.build());    //根据每个方法名也知道当前方法在设置什么参数
		
		return new Docket(DocumentationType.SWAGGER_2)
				.apiInfo(apiInfo())
				.select()
				//.apis(RequestHandlerSelectors.any())
				.apis(RequestHandlerSelectors.basePackage("com.dcit"))
				.paths(PathSelectors.any())
				.build()
				.globalOperationParameters(pars);
	}
	
	
	public ApiInfo apiInfo(){
		return new ApiInfoBuilder()
				.title("IOC-danger Restful APIs 测试")
				.contact(new Contact("SJZ DC-IT co. LTD", "www.hebdc.com", "352465677@qq.com"))
				.version("v1.0")
				.description("Restful Apis docs")
				.build();
	}
	
}
