package com.dcit.danger.controller;

import com.dcit.common.enums.ResultEnum;
import com.dcit.common.utils.ResultVOUtil;
import com.dcit.common.vo.ResultVO;
import com.dcit.danger.dto.AlarmInfoDTO;
import com.dcit.danger.dto.query.AccidentQuery;
import com.dcit.danger.dto.query.AlarmInfoQuery;
import com.dcit.danger.exception.IOCException;
import com.dcit.danger.model.AccidentStatistics;
import com.dcit.danger.service.AccidentStatisticsService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(value="事故信息API")
@RestController
@RequestMapping("accidentInfo")
@Slf4j
public class AccidentStatisticsController {
    @Autowired
    private AccidentStatisticsService accidentStatisticsService;

    @ApiOperation(value = "查询预警列表")
    @PostMapping("/getAccidentInfoList")
    public ResultVO<Object> selectListByEnterpriseId(@RequestBody AccidentQuery accidentQuery) {
        ModelMap modelMap = new ModelMap();
        try {

            PageHelper.offsetPage(accidentQuery.getPage(), accidentQuery.getSize());

            List<AccidentStatistics> dtoList = accidentStatisticsService.selectListByEnterpriseId(accidentQuery.getEnterpriseId(), accidentQuery.getFromdate());

            PageInfo<AccidentStatistics> pageInfoList = new PageInfo<AccidentStatistics>(dtoList);
            modelMap.addAttribute("pageData", pageInfoList.getList());
            modelMap.addAttribute("total", pageInfoList.getTotal());

        } catch (Exception e) {
            e.printStackTrace();
            log.error("【查询报警列表】操作出错,error={}", e.getMessage());
            throw new IOCException(ResultEnum.OPERATION_FAIL);
        }
        return ResultVOUtil.success(modelMap);
    }
}
