package com.dcit.danger.controller;


import com.dcit.common.enums.ResultEnum;
import com.dcit.common.utils.ResultVOUtil;
import com.dcit.common.vo.ResultVO;
import com.dcit.danger.dto.PlanInfoDTO;
import com.dcit.danger.dto.form.MaterialForm;
import com.dcit.danger.dto.query.MaterialQuery;
import com.dcit.danger.dto.query.PlanInfoQuery;
import com.dcit.danger.exception.IOCException;
import com.dcit.danger.model.MaterialInfo;
import com.dcit.danger.model.SnowFlakeUtil;
import com.dcit.danger.service.Materialservice;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(value="应急物资管理")
@RestController
@RequestMapping("material")
public class MaterialController {

    @Autowired
    private Materialservice materialservice;

    /**
     * 应急物资列表
     * @param planInfoQuery
     * @return
     */
    @ApiOperation(value = "应急物资列表")
    @PostMapping("/materialInfoList")
    public ResultVO<Object> materialInfoList(@RequestBody MaterialQuery materialQuery){

        ModelMap modelMap = new ModelMap();
        try {
            PageHelper.offsetPage(materialQuery.getPage(), materialQuery.getSize());
            List<Map<String,Object>> dtoList = materialservice.materialInfoList(materialQuery);

            PageInfo<Map<String,Object>> pageList = new PageInfo<Map<String,Object>>(dtoList);
            modelMap.addAttribute("pageData", pageList.getList());
            modelMap.addAttribute("total", pageList.getTotal());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultVOUtil.success(modelMap);

    }

    /**
     * 添加应急物资
     * @param materialForm
     * @return
     */
    @ApiOperation(value = "应急物资列表")
    @PostMapping("/addMaterInfo")
    public ResultVO<Object> addMaterInfo(@RequestBody MaterialForm materialForm){
        MaterialInfo materialInfo = new MaterialInfo(materialForm);
        if(materialInfo.getId()!=null){
            materialInfo.setUpdateTime(new Date());
            materialservice.updateMaterInfo(materialInfo);
        }else {
            materialInfo.setId(String.valueOf(SnowFlakeUtil.getFlowIdInstance().nextId()));
            materialInfo.setCreateTime(new Date());
            materialservice.addMaterInfo(materialInfo);
        }
        return ResultVOUtil.success();

    }

    @ApiOperation(value = "删除应急物资列表")
    @GetMapping("/deleteMaterial/{id}")
    public ResultVO<Object> deleteMaterial(@PathVariable String id){
        try {
            materialservice.deleteMaterial(id);
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOCException(ResultEnum.OPERATION_FAIL);
        }

        return ResultVOUtil.success();
    }
}
